package db

import (
	"context"
	"fmt"
	"github.com/caixw/lib.go/conv"
	"github.com/sirupsen/logrus"
	"gorm.io/gorm"
	"reflect"
	"youngee_m_api/model/common_model"
	"youngee_m_api/model/gorm_model"
	"youngee_m_api/util"
)

func GetFullProjectList(ctx context.Context, pageSize, pageNum int32, condition *common_model.ProjectCondition) ([]*gorm_model.ProjectInfo, int64, error) {
	db := GetReadDB(ctx)
	// 根据企业id过滤
	db = db.Debug().Model(gorm_model.ProjectInfo{})
	// 根据Project条件过滤
	conditionType := reflect.TypeOf(condition).Elem()
	conditionValue := reflect.ValueOf(condition).Elem()
	for i := 0; i < conditionType.NumField(); i++ {
		field := conditionType.Field(i)
		tag := field.Tag.Get("condition")
		value := conditionValue.FieldByName(field.Name)
		if tag == "project_status" && util.IsBlank(value) {
			db = db.Where(fmt.Sprintf("project_status != 1"))
		}
		if !util.IsBlank(value) && tag != "updated_at" && tag != "project_name" {
			db = db.Where(fmt.Sprintf("%s = ?", tag), value.Interface())
		}
		if tag == "updated_at" && value.Interface() != "0" {
			db = db.Where(fmt.Sprintf("%s > ?", tag), value.Interface())
		}
		if tag == "project_name" && !util.IsBlank(value) {
			db = db.Where(fmt.Sprintf("%s = ?", tag), value.Interface())
		}
	}
	// 查询总数
	var total int64
	var fullProjects []*gorm_model.ProjectInfo
	if err := db.Count(&total).Error; err != nil {
		logrus.WithContext(ctx).Errorf("[GetFullProjectList] error query mysql total, err:%+v", err)
		return nil, 0, err
	}
	// 查询该页数据
	limit := pageSize
	offset := pageSize * pageNum // assert pageNum start with 0
	err := db.Order("project_id").Limit(int(limit)).Offset(int(offset)).Find(&fullProjects).Error
	if err != nil {
		logrus.WithContext(ctx).Errorf("[GetFullProjectList] error query mysql total, err:%+v", err)
		return nil, 0, err
	}
	return fullProjects, total, nil
}

func GetProjectDetail(ctx context.Context, projectID int64) (*gorm_model.ProjectInfo, error) {
	db := GetReadDB(ctx)
	var ProjectDetail *gorm_model.ProjectInfo
	err := db.Where("project_id = ?", projectID).First(&ProjectDetail).Error
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return nil, nil
		} else {
			return nil, err
		}
	}
	return ProjectDetail, nil
}

func GetRecruitStrategys(ctx context.Context, ProjectID int64) ([]gorm_model.RecruitStrategy, error) {
	db := GetReadDB(ctx)
	RecruitStrategys := []gorm_model.RecruitStrategy{}
	err := db.Where("project_id=?", ProjectID).Find(&RecruitStrategys).Error
	if err != nil {
		return nil, err
	}
	return RecruitStrategys, nil
}

func GetProjectPhoto(ctx context.Context, ProjectID int64) ([]gorm_model.ProjectPhoto, error) {
	db := GetReadDB(ctx)
	var ProjectPhoto []gorm_model.ProjectPhoto
	err := db.Where("project_id=?", ProjectID).Find(&ProjectPhoto).Error
	if err != nil {
		return nil, err
	}
	return ProjectPhoto, nil
}

func UpdateProject(ctx context.Context, project gorm_model.ProjectInfo) (*int64, error) {
	db := GetReadDB(ctx)
	err := db.Model(&project).Updates(project).Error
	if err != nil {
		return nil, err
	}
	return &project.ProjectID, nil
}

func ApproveProject(ctx context.Context, projectId string,isApprove int64) (error error,message string) {
	project_id := conv.MustInt64(projectId,0)
	db := GetReadDB(ctx)
	projectInfo := gorm_model.ProjectInfo{}
	db = db.Debug().Model(gorm_model.ProjectInfo{}).Where("project_id = ?",project_id).First(&projectInfo)
	if isApprove == 1 &&  projectInfo.ProjectStatus == 2{
		err := db.Update("project_status", 4).Error
		message = "审核通过"
		if err != nil {
			logrus.Println("DB AutoUpdateStatus error :", err)
			return err,""
		}
	}else if isApprove == 2 &&  projectInfo.ProjectStatus == 2{
		err := db.Update("project_status", 8).Error
		message = "项目存在风险已禁止发布"
		if err != nil {
			logrus.Println("DB AutoUpdateStatus error :", err)
			return err,""
		}
	}else {
		return nil,"操作失败"
	}
	return nil,message
}
